<?php

namespace App\Http\Controllers\Api;

use App\Models\Question;
use App\Models\Choice;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Lesson;
use App\Models\Module;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use App\Mail\AdminMarksMail;


class ChoiceController extends Controller
{

    public function store(Request $request, $questionId)
    {
        $data = $request->validate([
            'options' => 'required|array|min:4',
            'options.*.content' => 'required|string',
            'correct_option' => 'required|integer|between:0,3', // Assuming options are indexed from 0 to 3
        ]);

        $question = Question::findOrFail($questionId);

        // Create choices from the options array
        $choices = [];
        foreach ($data['options'] as $index => $option) {
            $isCorrect = $index == $data['correct_option'];
            $choices[] = ['content' => $option['content'], 'is_correct' => $isCorrect];
        }

        // Save choices to the database
        $question->choices()->createMany($choices);

        return response()->json(['message' => 'Choices created successfully'], 201);
    }


    public function getCorrectAnswer($questionId)
    {
        $question = Question::findOrFail($questionId);

        $correctAnswer = $question->choices()->where('is_correct', true)->first();

        if ($correctAnswer) {
            return response()->json(['correct_answer' => $correctAnswer], 200);
        } else {
            return response()->json(['message' => 'Correct answer not found for the given question'], 404);
        }
    }


    public function submitUserResponse(Request $request, $questionId)
    {
        $data = $request->validate([
            'choice_id' => 'required|exists:choices,id',
        ]);

        $user = auth()->user(); // Assuming you are using Laravel's built-in authentication
        $question = Question::findOrFail($questionId);

        // Store user response
        $user->responses()->create([
            'question_id' => $question->id,
            'choice_id' => $data['choice_id'],
        ]);

        return response()->json(['message' => 'User response submitted successfully'], 201);
    }




/** Code by San || Added Quiz Retake Option  */
public function submitUserResponses(Request $request, $quizId)
{     
    try {
        $requestData = $request->validate([
            'responses' => 'required|array|min:1',
            'responses.*.question_id' => 'required|exists:questions,id',
            'responses.*.choice_id' => 'required|exists:choices,id',
        ]);

        $user = auth()->user();
        $userId = $user->id;
        $adminId = $user->admin_id;
        $admin = User::where('id', $adminId)->first();
        $admin_email = $admin->email;
        $assignedCourses = $user->courses;

        $quiz =  \DB::table('quizzes')
        ->where('id', $quizId)
        ->first();
        $quizName = $quiz->title;

        if (!$user) {
            Log::error('User not authenticated');
            return response()->json(['error' => 'User not authenticated'], 401);
        }

        $attempt = DB::table('quiz_attempts')
            ->where('user_id', $user->id)
            ->where('quiz_id', $quizId)
            ->first();

        if ($attempt && $attempt->attempt_count >= 3) {
            return response()->json(['message' => 'Maximum number of attempts reached'], 403);
        }

        // dd($attempt);

        if ($attempt) {
            DB::table('quiz_attempts')
                ->where('user_id', $user->id)
                ->where('quiz_id', $quizId)
                ->increment('attempt_count');
                $no_of_attemp = $attempt->attempt_count ;

        } else {
            DB::table('quiz_attempts')->insert([
                'user_id' => $user->id,
                'quiz_id' => $quizId,
                'attempt_count' => 1,
                'created_at' => now(),
                'updated_at' => now(),
            ]);
            $no_of_attemp = 0; // First attempt
        }

        $feedback = [];
        $no_of_quest = 1;
        $correctans = 0;
        foreach ($requestData['responses'] as $response) {
            $question = Question::findOrFail($response['question_id']);
            $correct = Choice::where('is_correct', 1)->where('question_id', $response['question_id'])->first(['content']);

            $answer = $correct->is_correct;

            // Fetch the choice details using the ID
        
            $correctChoiceId = $question->choices()->where('is_correct', true)->value('id');
            $isCorrect = $response['choice_id'] == $correctChoiceId;
            if($isCorrect == true){
                $correctans = $correctans+1;
            }else{
                $correctans ;
            }

            $user->responses()->create([
                'question_id' => $question->id,
                'choice_id' => $response['choice_id'],
            ]);

            $feedback[] = [
                'question_id' => $question->id,
                'question' => $question->content,
                'is_correct' => $isCorrect,
                'no_of_attempt' => $no_of_attemp,
                'correct_answer' => $correct,
                'No_of_ques' => $no_of_quest,
                'no_of_correct' => $correctans,
                'percentage' => ($correctans / $no_of_quest) * 100
                // 'details' => $correctChoiceDetails,
            ];
            $no_of_quest ++;
        }

        $no_of_quest = $no_of_quest-1;

        $percentage = ($correctans / $no_of_quest) * 100;

        $attempt_no = $no_of_attemp + 1;


        $user->quizzes()->updateExistingPivot($quizId, ['status' => 'Retake']);

        // dd($user);

        // Update lesson, module, and course statuses if applicable
        // Ensure these operations are optimized and handle edge cases gracefully
        Mail::to($admin_email)->send(new AdminMarksMail($user,$correctans,$no_of_quest,$percentage, $attempt_no, $quizName));

        return response()->json(['message' => 'User responses submitted successfully', 'feedback' => $feedback], 201);
    } catch (\Exception $e) {
        Log::error('Exception: ' . $e->getMessage());
        return response()->json(['error' => 'Please attempt atleast 1 question'], 500);
    }
}
/** Code by San || Added Quiz Retake Option || Closed */


public function calculateUserScore($userId)
{
    $user = User::findOrFail($userId);

    $adminId = $user->admin_id;
    
    $admin = User::where('id', $adminId)->first();
    $admin_email = $admin->email;


    $userResponses = $user->responses()->with('choice', 'question')->get();

    $totalQuestions = count($userResponses);
    $correctAnswers = 0;

    // Store detailed feedback for each question
    $detailedFeedback = [];

    foreach ($userResponses as $response) {
        if ($response->choice && $response->question) {
            if ($response->choice->is_correct) {
                $correctAnswers++;
            } else {
                // Store detailed feedback for incorrect responses
                $detailedFeedback[] = [
                    'question' => $response->question->text,
                    'selected_choice' => $response->choice->text,
                    'correct_choice' => $response->question->correctChoice()->text
                ];
            }
        } else {
            // Log the error for investigation
            \Log::error("Choice or question is null for response ID: {$response->id}");
        }
    }

    $percentage = ($totalQuestions > 0) ? ($correctAnswers / $totalQuestions) * 100 : 0;

    // Mail::to($admin_email)->send(new AdminMarksMail($user,$correctAnswers,$totalQuestions,$percentage, $detailedFeedback));
 

    return response()->json([
        'message' => 'User Score',
        'correct_answers' => $correctAnswers,
        'total_questions' => $totalQuestions,
        'percentage' => $percentage,
        'detailed_feedback' => $detailedFeedback,
        // 'send' => $send
    ]);
}



}