<?php


namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Lesson;
use App\Models\Module;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Validator;
use Illuminate\Log\Logger;
use App\Http\Requests\LessonCreateRequest;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\Auth;
use App\Helpers\AdminHelper;

class LessonController extends Controller
{


public function lessionCount()
{
    $admin = Auth::user();

    if ($admin->role !== 1 && $admin->role !== 2) {
        return response()->json(['message' => 'Unauthorized. Only admins can view lesson count.'], 403);
    }

    $assignedId = AdminHelper::getAssignedId();

    // Determine the admin ID based on the value of assignedId
    $adminId = ($assignedId === "null") ? $admin->id : $assignedId;

    $adminAccount = User::find($adminId); 
    $lessonCount = $adminAccount->createdCourses()->with('modules.lessons')->get()->flatMap(function ($course) {
        return $course->modules->flatMap->lessons;
    })->count();

    // $assignadmins = $admin->accountadmins()->distinct()->pluck('account_ids')->toArray();
    // if($assignadmins){
    //     foreach ($assignadmins as $assignadmin){
    //         $adminAccount = User::find($assignadmin);
    //         if($adminAccount){
    //             $lessonCount += $adminAccount->createdCourses()->with('modules.lessons')->get()->flatMap(function ($course) {
    //                 return $course->modules->flatMap->lessons;
    //             })->count();
    //         }
    //     }
    // }
    return response()->json(['lesson_count' => $lessonCount]);
}



public function createLesson(Request $request)
{

    $validationRules = [
        'title' => [
            'required',
            'string',
            Rule::unique('lessons')->where(function ($query) use ($request) {
                return $query->where('module_id', $request->input('module_id'));
            }),
        ],
        'content' => 'string|nullable',
        'module_id' => ['required', 'exists:module,id'],
        'order' => 'required|integer',
        'video_path' => 'required|string',
    ];

    $validationMessages = [
        'title.required' => 'The title is required.',
        'title.string' => 'The title must be a string.',
        'title.unique' => 'The title has already been taken for this module.',
        // 'content.required' => 'The content is required.',
        'module_id.required' => 'The module ID is required.',
        'module_id.exists' => 'The selected module is invalid.',
        'order.required' => 'The order is required.',
        'order.integer' => 'The order must be an integer.',
    ]; 

    try {
        $validatedData = $request->validate($validationRules, $validationMessages);
     
        if ($request->hasFile('video_path')) {
            $video = $request->file('video_path');
            $videoFileName = time() . '_' . $video->getClientOriginalName();
            $video->move(public_path('videos'), $videoFileName);
            $videoPath = $video;
            $validatedData['video_path'] = $videoPath;
            $validatedData['video_filename'] = $videoFileName;
        }
    
        $lesson = Lesson::create($validatedData);

        return response()->json([
            'message' => 'Lesson created successfully',
            'lesson' => $lesson,
        ], 201);
        
    } catch (ValidationException $e) {
        $firstErrorMessage = collect($e->errors())->flatten()->first();
        return response()->json([
            'message' => $firstErrorMessage, 
            'errors' => $e->errors(),
        ], 422);
    }


}

public function viewLessons()
    {
        try {
            // Retrieve all lessons
            $lessons = Lesson::all();

            return response()->json(['message' => 'All lessons', 'lessons' => $lessons], 200);
        } catch (\Exception $e) {
            // Log the exception details
            \Log::error('Exception occurred: ' . $e->getMessage());

            // Handle database operation exception
            return response()->json(['error' => 'Failed to retrieve lessons'], 500);
        }
    }

// one lesson view through id 
public function viewLessonAdmin($id)
    {
        try {
            // Retrieve a single lesson based on the ID
            $lesson = Lesson::find($id);

            if (!$lesson) {
                return response()->json(['error' => 'Lesson not found'], 404);
            }

            return response()->json(['message' => 'Lesson found', 'lesson' => $lesson], 200);
        } catch (\Exception $e) {
            // Log the exception details
            \Log::error('Exception occurred: ' . $e->getMessage());

            // Handle database operation exception
            return response()->json(['error' => 'Failed to retrieve lesson'], 500);
        }
    }


public function viewLesson(Request $request, $id)
{
    // Retrieve authenticated user
    $user = Auth::user();

    if (!$user) {
        return response()->json(['error' => 'User not authenticated'], 401);
    }


        $lesson = Lesson::find($id);

        if (!$lesson) {
            return response()->json(['error' => 'Lesson not found'], 404);
        }

        $assignedLesson = $user->lessons()->where('lesson_id', $lesson->id)->first();

        if (!$assignedLesson) {
            return response()->json(['error' => 'Lesson not assigned to user'], 404);
        }

        // Update the task status to "In Progress" if not already in progress
        if ($assignedLesson->pivot->status !== 'In Progress') {
            $assignedLesson->pivot->status = 'In Progress';
            $assignedLesson->pivot->save();
        }

    //         // Retrieve lessons for the module
    // $lessons = $lesson->lessons;

    return response()->json([
        'message' => 'Lesson status updated to In Progress successfully',
        'lesson' => $lesson
    ], 200);
}




public function viewLessonsForModuleAdmin($moduleId)
    {
        $module = Module::find($moduleId);

        if (!$module) {
            return response()->json(['message' => 'module not found'], 404);
        }

        $lessons = $module->lessons;

        return response()->json(['message' => 'Lessons for the module', 'lessons' => $lessons], 200);
    }


public function viewLessonsForModule(Request $request, $moduleId)
{
    // Retrieve authenticated user
    $user = Auth::user();

    if (!$user) {
        return response()->json(['error' => 'User not authenticated'], 401);
    }

    // Retrieve the module associated with the module ID
    $module = Module::find($moduleId);

    if (!$module) {
        return response()->json(['error' => 'Module not found'], 404);
    }


    $assignedModule = $user->modules()->where('module_id', $module->id)->first();

    if (!$assignedModule) {
        return response()->json(['error' => 'Module not assigned to user'], 404);
    }

    // Update the task status to "In Progress" if not already in progress
    if ($assignedModule->pivot->status !== 'In Progress') {
        $assignedModule->pivot->status = 'In Progress';
        $assignedModule->pivot->save();
    }

    // Retrieve lessons for the module
    $lessons = $module->lessons;

    return response()->json([
        'message' => 'Module status updated to In Progress successfully',
        'lessons' => $lessons
    ], 200);
}


public function editLesson(Request $request, $id)
{
    $lesson = Lesson::find($id);

    if (!$lesson) {
        return response()->json(['error' => 'Lesson not found'], 404);
    }

    // Custom error messages
    $messages = [
        'title.string' => 'The title must be a string.',
        'title.unique' => 'The title is already taken for the selected module.',
        'content.string' => 'The content must be a string.',
        'order.required' => 'The order is required.',
        'order.integer' => 'The order must be an integer.',
        'video_path.required' => 'The video path is required.',
        'video_path.string' => 'The video path must be a string.',
    ];

    try {
        // Validate the request data
        $validatedData = $request->validate([
            'title' => [
                'string',
                Rule::unique('lessons')->where(function ($query) use ($lesson) {
                    return $query->where('module_id', $lesson->module_id);
                })->ignore($lesson->id),
            ],
            'content' => 'string|nullable',
            'order' => 'integer',
            'video_path' => 'required|string',
        ], $messages); // Pass the custom messages here

        // Update the lesson with the validated data
        $updateResult = $lesson->update($validatedData);

        if ($updateResult) {
            return response()->json(['message' => 'Lesson updated successfully', 'lesson' => $lesson], 200);
        } else {
            return response()->json(['error' => 'Failed to update lesson'], 500);
        }
        
    } catch (ValidationException $e) {
        // Capture the first error message
        $firstErrorMessage = collect($e->errors())->flatten()->first();

        return response()->json([
            'message' => $firstErrorMessage,
            'errors' => $e->errors(),
        ], 422);
    } catch (\Exception $e) {
        // Handle any other exceptions
        return response()->json([
            'error' => 'An error occurred: ' . $e->getMessage(),
        ], 500);
    }
}






    public function deleteLesson($id)
    {
        try {
            // Check if the lesson exists
            $lesson = Lesson::find($id);

            if (!$lesson) {
                return response()->json(['error' => 'Lesson not found'], 404);
            }

            // Delete the lesson
            $lesson->delete();

            return response()->json(['message' => 'Lesson deleted successfully'], 200);
        } catch (\Exception $e) {
            // Log the exception details
            \Log::error('Exception occurred: ' . $e->getMessage());

            // Handle database operation exception
            return response()->json(['error' => 'Failed to delete lesson'], 500);
        }
    }
}
