<?php

namespace App\Http\Controllers\Api;

use App\Models\Quiz;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Lesson;
use App\Models\Module;
use App\Models\NewCourse;
use Illuminate\Validation\Rule;
use App\Models\User;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\ValidationException;


class QuizController extends Controller
{

public function createQuiz(Request $request)
{
    try {
        // Check for blank fields
        $blankFields = collect($request->all())->filter(function ($value) {
            return $value === '';
        });
    
        if ($blankFields->isNotEmpty()) {
            return response()->json(['message' => 'Blank fields detected'], 422);
        }
    
        // Validate the request data, including custom rule for quiz uniqueness in the lesson
        $validatedData = $request->validate([
            'title' => [
                'required',
                'string',
                Rule::unique('quizzes')->where(function ($query) use ($request) {
                    return $query->where('lesson_id', $request->input('lesson_id'));
                }),
            ],
            'lesson_id' => ['required', 'exists:lessons,id'],
        ], [
            'title.string' => 'The title must be a string.',
            'title.unique' => 'The quiz title is already taken.',
        ]);
    
        // Create the quiz
        $quiz = Quiz::create($validatedData);
    
        // Attach the newly created quiz to users who are enrolled in the associated course
        $course = Lesson::findOrFail($request->input('lesson_id'))->module->course;
        $users = $course->users;
    
        foreach ($users as $user) {
            // Check if the quiz is not already assigned to the user
            if (!$user->quizzes->contains($quiz->id)) {
                // Attach the quiz to the user with the default status 'Not Started'
                $user->quizzes()->attach($quiz, ['status' => 'Not Started']);
            }
        }
    
        return response()->json([
            'message' => 'Quiz created successfully',
            'quiz' => $quiz,
        ], 201);
    
    } catch (\Illuminate\Validation\ValidationException $e) {
        // Handle validation exception and format the output
        $errors = $e->errors();
        $formattedErrors = [];
    
        // Get the first error message from the validation errors
        $firstErrorMessage = collect($errors)->flatten()->first();
    
        return response()->json([
            'message' => $firstErrorMessage,  // Set the first error message here
            'details' => $errors,             // Return the full error structure here
        ], 422);
    
    } catch (\Exception $e) {
        // Log the exception for debugging purposes
        \Log::error('Failed to create quiz: ' . $e->getMessage());
    
        // Return a generic error message
        return response()->json(['message' => 'Failed to create quiz. Please try again later.'], 500);
    }
        
}

public function editQuiz(Request $request, $id)
{
    $quiz = Quiz::find($id);

    if (!$quiz) {
        return response()->json(['error' => 'Quiz not found'], 404);
    }

    // Custom error messages
    $messages = [
        'title.string' => 'The title must be a string.',
        'title.unique' => 'The quiz title is already taken for this lesson.',
    ];

    try {
        $validatedData = $request->validate([
            'title' => [
                'string',
                Rule::unique('quizzes')->where(function ($query) use ($quiz) {
                    return $query->where('lesson_id', $quiz->lesson_id);
                })->ignore($quiz->id),
            ],
        ], $messages); 

        $quiz->update($validatedData);

        return response()->json(['message' => 'Quiz updated successfully', 'quiz' => $quiz], 200);
        
    } catch (ValidationException $e) {
       
        $firstErrorMessage = collect($e->errors())->flatten()->first();

        return response()->json([
            'message' => $firstErrorMessage,
            'errors' => $e->errors(),
        ], 422);
    } catch (\Exception $e) {
      
        return response()->json([
            'error' => 'An error occurred: ' . $e->getMessage(),
        ], 500);
    }
}

    public function deleteQuiz($id)
    {
        try {
            // Check if the quiz exists
            $quiz = Quiz::find($id);

            if (!$quiz) {
                return response()->json(['error' => 'Quiz not found'], 404);
            }

            // Delete the quiz
            $quiz->delete();

            return response()->json(['message' => 'Quiz deleted successfully'], 200);
        } catch (\Exception $e) {
            // Log the exception details
            \Log::error('Exception occurred: ' . $e->getMessage());

            // Handle database operation exception
            return response()->json(['error' => 'Failed to delete quiz'], 500);
        }
    }

    public function viewQuizzes()
    { 
        try {
            // Retrieve all quizzes
            $quizzes = Quiz::all();

            return response()->json(['message' => 'All quizzes', 'quizzes' => $quizzes], 200);
        } catch (\Exception $e) {

            // Handle database operation exception
            return response()->json(['error' => 'Failed to retrieve quizzes'], 500);
        }
    }


    public function viewQuiz($id)
    {
        try {
            // Retrieve a single quiz based on the ID
            $quiz = Quiz::find($id);

            if (!$quiz) {
                return response()->json(['error' => 'Quiz not found'], 404);
            }

            return response()->json(['message' => 'Quiz found', 'quiz' => $quiz], 200);
        } catch (\Exception $e) {

            // Handle database operation exception
            return response()->json(['error' => 'Failed to retrieve quiz'], 500);
        }
    }



    /** Code by San || Quizz List With Retake Count */
    public function viewQuizzesForLessonwithstatus($userId, $lessonId)
    {
        $user = User::find($userId);
        if (!$user) {
            return response()->json(['message' => 'User not found'], 404);
        }
        $lesson = Lesson::find($lessonId);
        if (!$lesson) {
            return response()->json(['message' => 'Lesson not found'], 404);
        }
        $quizzes = $lesson->quizzes()->whereHas('users', function ($query) use ($userId) {
            $query->where('users.id', $userId);
        })->get();
        if ($quizzes->isNotEmpty()) {
            $quizzesData = $quizzes->map(function ($quiz) use ($user) {
                $pivot = $user->quizzes->find($quiz->id)->pivot;
            
                $attemptCount = \DB::table('quiz_attempts')
                    ->where('user_id', $user->id)
                    ->where('quiz_id', $quiz->id)
                    ->value('attempt_count');
    
                $retakeStatus = ($attemptCount >= 3) ? 'No retake available' : 'Can retake';
    
                return [
                    'id' => $quiz->id,
                    'title' => $quiz->title,
                    'status' => $pivot->status,
                    'retake_count' => $attemptCount ?? 0,
                    'retake_status' => $retakeStatus,
                    'created_at' => $quiz->created_at->format('m/d/Y'),
                    'updated_at' => $quiz->updated_at->format('m/d/Y'),
                ];
            });
    
            return response()->json(['message' => 'Quizzes for the lesson', 'quizzes' => $quizzesData], 200);
        } else {
            return response()->json(['message' => 'No quizzes found for the lesson'], 404);
        }
    }
    /** Code by San || Quizz List With Retake Count || Closed*/

    public function viewQuizzesForLesson($lessonId)
    {
      
        \Log::info('viewQuizzesForLesson - Start', ['lesson_id' => $lessonId]);
        $lesson = Lesson::find($lessonId);
        if (!$lesson) {
          
            \Log::warning('Lesson not found', ['lesson_id' => $lessonId]);
            return response()->json(['message' => 'Lesson not found'], 404);
        }

        $quizzess = $lesson->quizzes;
        $quizzes = $quizzess->map(function ($quiz) {
            return [
                'id' => $quiz->id,
                'title' => $quiz->title,
                'created_at' => $quiz->created_at->format('m/d/Y'),
                'updated_at' => $quiz->updated_at->format('m/d/Y'),
            ];
        });
       
        \Log::info('Quizzes retrieved', ['lesson_id' => $lessonId, 'quizzes' => $quizzes]);
        return response()->json(['message' => 'Quizzes for the lesson', 'quizzes' => $quizzes], 200);
    }
    public function viewQuizzesForLessonAdmin($lessonId)
    {
      
        \Log::info('viewQuizzesForLesson - Start', ['lesson_id' => $lessonId]);
        $lesson = Lesson::find($lessonId);
        if (!$lesson) {
          
            \Log::warning('Lesson not found', ['lesson_id' => $lessonId]);
            return response()->json(['message' => 'Lesson not found'], 404);
        }

        $quizzess = $lesson->quizzes;
        $quizzes = $quizzess->map(function ($quiz) {
            return [
                'id' => $quiz->id,
                'title' => $quiz->title,
                'created_at' => $quiz->created_at->format('m/d/Y'),
                'updated_at' => $quiz->updated_at->format('m/d/Y'),
            ];
        });
       
        \Log::info('Quizzes retrieved', ['lesson_id' => $lessonId, 'quizzes' => $quizzes]);
        return response()->json(['message' => 'Quizzes for the lesson', 'quizzes' => $quizzes], 200);
    }

// public function getQuestionsWithChoices($quizId)
// {
    
//     $user = auth()->user();
//     $quiz = Quiz::findOrFail($quizId);
//     $lesson = Lesson::findOrFail($quizId);
//     $module = Module::findOrFail($lesson->module_id);
//     $course = NewCourse::findOrFail($module->course_id);
//     $courseId = $course->id;
//     $attempt = DB::table('quiz_attempts')
//             ->where('user_id', $user->id)
//             ->where('quiz_id', $quizId)
//             ->first();

//     if($attempt){
//     $attempt_count = $attempt->attempt_count;
//     }else{
//         $attempt_count = 0 ;
//     }

//     $questionsWithChoices = $quiz->questions()->with('choices')->get();

//     return response()->json(['questions' => $questionsWithChoices, 'course_id' => $courseId, 'no_of_attempt' => $attempt_count ], 200);
// }



    public function getQuestionsWithChoices($quizId)
{
    $user = auth()->user();

    // Fetch the quiz by its ID
    $quiz = Quiz::findOrFail($quizId);

    // Assuming the Quiz has a relationship with Lesson
    $lesson = $quiz->lesson; // Adjust this if the relationship is different

    // Fetch the module based on the lesson's module_id
    $module = Module::findOrFail($lesson->module_id);

    // Fetch the course based on the module's course_id
    $course = NewCourse::findOrFail($module->course_id);
    $courseId = $course->id;

    // Check if there's an attempt by the user for this quiz
    $attempt = DB::table('quiz_attempts')
        ->where('user_id', $user->id)
        ->where('quiz_id', $quizId)
        ->first();

    $attempt_count = $attempt ? $attempt->attempt_count : 0;

    // Fetch the questions with their choices
    $questionsWithChoices = $quiz->questions()->with('choices')->get();

    return response()->json([
        'questions' => $questionsWithChoices,
        'course_id' => $courseId,
        'no_of_attempt' => $attempt_count
    ], 200);
}



    

//     public function getQuestionsWithChoices($quizId)
// {
//     $quiz = Quiz::findOrFail($quizId);
//     $questionsWithChoices = $quiz->questions()->with('choices')->get();

//     return response()->json(['questions' => $questionsWithChoices], 200);
// }

}
